#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <sys/time.h>

#include "convert.h"

enum test_case {
	BASE,
	SSE,
};

static void convert_si16_ps(float *out, short *in, int len)
{
        for (int i = 0; i < len; i++)
                out[i] = in[i];
}

static void convert_scale_ps_si16(short *out, float *in, float scale, int len)
{
        for (int i = 0; i < len; i++)
                out[i] = in[i] * scale;
}

static int verify_short_float_short(short *out, float *in, float scale, int num)
{
	for (int i = 0; i < num; i++) {
		if (abs(out[i] - (int) (in[i] * scale)) > 1)
			return -1;
	}

	return 0;
}

static int verify_short_float(float *out, short *in, int num)
{
	for (int i = 0; i < num; i++) {
		if (fabs(out[i] - (float) in[i]) > 1.0f)
			return -1;
	}

	return 0;
}

static const char *get_test_name(int test)
{
	switch (test) {
	case BASE:
		return "Base";
	case SSE:
		return "SSE";
	}

	return "";
}

static double test_float_short(int len, int num, int test)
{
	float *in, scale = 32700.0f;
	short *out;
	const char *name = get_test_name(test);
	double elapsed;
	struct timeval tv0, tv1;
	void (*convert_func)(short *, float *, float, int);

	if (test == BASE)
		convert_func = convert_scale_ps_si16;
	else if (test == SSE)
		convert_func = convert_float_short;
	else
		return 0.0f;

	in = malloc(num * len * sizeof(float));
	out = malloc(num * len * sizeof(short));

	for (int i = 0; i < num * len; i++)
		in[i] = (float) rand() / (float) RAND_MAX;

	gettimeofday(&tv0, NULL);

	for (int i = 0; i < num; i++)
		convert_func(&out[i * len], &in[i * len], scale, len);

	gettimeofday(&tv1, NULL);

	printf("Verifying results... ");
	if (verify_short_float_short(out, in, scale, num) < 0)
		printf("%s results do no match\n", name);
	else
		printf("Ok\n");

	elapsed = (tv1.tv_sec - tv0.tv_sec);
	elapsed += (tv1.tv_usec - tv0.tv_usec) / 1e6;

	printf("%s elapsed time... %f secs\n", name, elapsed);

	free(in);
	free(out);

	return elapsed;
}

static double test_short_float(int len, int num, int test)
{
	short *in;
	float *out;
	const char *name = get_test_name(test);
	double elapsed;
	struct timeval tv0, tv1;
	void (*convert_func)(float *, short *, int);

	if (test == BASE)
		convert_func = convert_si16_ps;
	else if (test == SSE)
		convert_func = convert_short_float;
	else
		return 0.0f;

	in = malloc(num * len * sizeof(short));
	out = malloc(num * len * sizeof(float));

	for (int i = 0; i < num * len; i++)
		in[i] = rand() % 32768;

	gettimeofday(&tv0, NULL);

	for (int i = 0; i < num; i++)
		convert_func(&out[i * len], &in[i * len], len);

	gettimeofday(&tv1, NULL);

	printf("Verifying results... ");
	if (verify_short_float(out, in, len * num) < 0)
		printf("%s results do no match\n", name);
	else
		printf("Ok\n");

	elapsed = (tv1.tv_sec - tv0.tv_sec);
	elapsed += (tv1.tv_usec - tv0.tv_usec) / 1e6;
	printf("%s elapsed time... %f secs\n", name, elapsed);

	free(in);
	free(out);

	return elapsed;
}

int main(int argc, char **argv)
{
	int len[2] = { 1248, 312 };
	int num = 100000;
	double time0, time1;

	printf("Floating point to integer conversions\n");
	for (int i = 0; i < 2; i++) {
		printf("Testing %i iterations of %i values\n", num, len[i]);
		time0 = test_float_short(len[i], num, BASE);
		time1 = test_float_short(len[i], num, SSE);
		printf("Quotient %f\n", time0 / time1);
		printf("\n");
	}

	printf("Integer to floating point conversions\n");
	for (int i = 0; i < 2; i++) {
		printf("Testing %i iterations of %i values\n", num, len[i]);
		time0 = test_short_float(len[i], num, BASE);
		time1 = test_short_float(len[i], num, SSE);
		printf("Quotient %f\n", time0 / time1);
		if (i == 1)
			printf("*SSE 8*N integer to float not implemented*\n");
		printf("\n");
	}
}
