#ifndef _LTE_VITERBI_H_
#define _LTE_VITERBI_H_

#include <stdint.h>

struct lte_vdecoder;

/* Convolutional code descriptor
 *     n - rate
 *     k - constraint length
 *     len - trellis length
 *     rgen - recursive generator polynomial in octal
 *     gen - generator polynomial in octal
 *     punc - puncturing matrix
 */
struct lte_conv_code {
	int n;
	int k;
	int len;
	unsigned rgen;
	unsigned gen[3];
	int *punc;
};

struct lte_vdecoder *lte_alloc_vdec(const struct lte_conv_code *code);
int lte_conv_decode(struct lte_vdecoder *dec,
		    int8_t *in, int *punc, uint8_t *out, int len);
void lte_free_vdec(struct lte_vdecoder *dec);

void gen_branch_metrics_k5_n2(int8_t *val, int16_t *outputs, int16_t *metrics);
void gen_branch_metrics_k5_n3(int8_t *val, int16_t *outputs, int16_t *metrics);
void gen_branch_metrics_k5_n4(int8_t *val, int16_t *outputs, int16_t *metrics);
void gen_branch_metrics_k7_n2(int8_t *val, int16_t *outputs, int16_t *metrics);
void gen_branch_metrics_k7_n3(int8_t *val, int16_t *outputs, int16_t *metrics);
void gen_branch_metrics_k7_n4(int8_t *val, int16_t *outputs, int16_t *metrics);

int gen_path_metrics_k5(int16_t *sums, int16_t *metrics, int16_t *paths, int norm);
int gen_path_metrics_k7(int16_t *sums, int16_t *metrics, int16_t *paths, int norm);

#endif /* _LTE_VITERBI_H_ */
